<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

	class User_model extends CI_Model
	{
		public function __construct()
		{
			parent::__construct();
			$this->load->database();
			$this->load->library('user_auth');
			// $this->output->enable_profiler(TRUE);
		}

		public function operation_max_min(){
			$this->db->from('operations AS op');
			$this->db->select(
				'MAX(op.IdOperation) as LatestOperation, MIN(op.IdOperation) as EarliestOperation'
			);

			$result = $this->db->get();
			
			return $result->result();
		}

		public function get_all_users(){
			$query = $this->db->get('kasir');

			return $query->result(); //The result is an array of objects, each row = an object
		}

		public function change_password($id){
			$password = $this->input->post('password');
			
			$data = array(
				'password' => $this->user_auth->hash_password($password)
			);

			$this->db->where('IdGrantor', $id);
			$this->db->update('kasir_grantor', $data);
		}

		public function get_user_data($data){
			$this->db->where('IdGrantor', $data);
			
			return $this->db->get('kasir_grantor')->row();
		}

		public function get_agen($data){
			$this->db->from('kasir');
			$this->db->where('IdKasirInduk', $data['id_kasir_induk']);
			$this->db->where('Level', $data['level']);
			$result = $this->db->get()->result();

			foreach($result as &$item)
			{
				$data = array(
                    'id_kasir_induk' => $item->IdKasir,
                    'level' => 2
                ); 

				if (isset($this->count_anak_agen($data)[0]['total'])) {
					$item->TotalAnak = $this->count_anak_agen($data)[0]['total'];
				}else{
					$item->TotalAnak = 0;
				}
			}

			return $result;
		}

		public function get_id_agen($data){
			$this->db->from('kasir');
			$this->db->where_in('IdKasirInduk', $data['id_kasir_induk']);
			$this->db->select('IdKasir');
			$result = $this->flatten($this->db->get()->result_array());

			return $result;
		}

		// Get anak 1 level dibawah
		public function get_anak_agen($data){
			$this->db->from('kasir');
			$this->db->where('IdKasirInduk', $data['id_kasir_induk']);
			$result = $this->db->get()->result();

			foreach($result as &$item)
			{
				$data = array(
                    'id_kasir_induk' => $item->IdKasir,
                );

				if (isset($this->count_player($data)[0]['total'])) {
					$item->TotalAnak = $this->count_player($data)[0]['total'];
				}
				else{
					$item->TotalAnak = 0;
				}
			}

			return $result;
		}

		public function get_all_subagen($data){
			$this->db->from('kasir');
			$this->db->select('kasir.IdKasir');
			$this->db->distinct();
			$this->db->where('IdKasirInduk', $data['id_kasir_induk']);

			// Ambil ID Agen dan di flatenned menjadi array biasa
			$result = $this->flatten($this->db->get()->result_array());

			return $result;
		}

		public function get_all_agen($data){
			$this->db->from('kasir');
			$this->db->where('IdKasirInduk', $data['id_kasir_induk']);
			$this->db->order_by('Nama', 'asc');
			$result = $this->db->get()->result();

			return $result;
		}

		public function get_all_subagen_dan_player($data){
			$this->db->from('kasir');
			$this->db->select('IdKasir');
			$this->db->distinct();
			$this->db->where('IdKasirInduk', $data['id_kasir_induk']);

			// Ambil ID Agen dan di flatenned menjadi array biasa
			$result = $this->flatten($this->db->get()->result_array());

			// Search id masing2 player berdasarkan ID kasir
			$player_id_result = $this->get_id_anak_agen_flattened($result);

			return $player_id_result;
		}

		public function get_id_anak_agen_flattened($data){
			$this->db->select('IdPlayer');
			$this->db->from('players');
			$this->db->where_in('IdKasir', $data);
			$this->db->distinct();

			$result = $this->flatten($this->db->get()->result_array());

			return $result;
		}

		// Hitung jumlah anak agen
		public function count_anak_agen($data){
			$this->db->select('IdKasirInduk, COUNT(IdKasirInduk) as total');
			$this->db->group_by('IdKasirInduk'); 
			$this->db->from('kasir');
			$this->db->where('IdKasirInduk', $data['id_kasir_induk']);
			$this->db->where('Level', $data['level'] + 1);

			$result = $this->db->get()->result_array();
			
			return $result;
		}

		// Hitung jumplah player di bawah kasir
		public function count_player($data){
			$this->db->select('IdKasir, COUNT(IdKasir) as total');
			$this->db->group_by('IdKasir'); 
			$this->db->from('players');
			$this->db->where('IdKasir', $data['id_kasir_induk']);

			$result = $this->db->get()->result_array();

			return $result;
		}		

		// Ambil semua player yang langsung dibawah kasir
		public function get_player($data){		
			$this->db->from('players');
			$this->db->where('IdKasir', $data['id_kasir_induk']);
			$this->db->order_by("NickName", "asc");
			$result = $this->db->get()->result();

			return $result;
		}

		public function get_playerid_flatten($data, $select = FALSE){
			if ($select != FALSE) {
				$this->db->select($select);
			}
		
			$this->db->from('players');
			$this->db->where('IdKasir', $data['id_kasir_induk']);
			$this->db->select('accounts.IdAccount');

			$this->db->join('accounts', 'accounts.IdPlayer = players.IdPlayer');

			$result = $this->flatten($this->db->get()->result_array());
			// $result = $this->db->get()->result();

			return $result;	
		}

		public function get_player_account_flatten($data){

			$this->db->from('players');
			$this->db->where_in('IdKasir', $data['id_kasir_induk']);
			$this->db->select('accounts.IdAccount');

			$this->db->join('accounts', 'accounts.IdPlayer = players.IdPlayer');

			$result = $this->flatten($this->db->get()->result_array());

			return $result;	
		}

		// Ambil semua player dibawah agen.
		public function get_subagen_and_players($data){
			$this->db->from('kasir');
			$this->db->where('IdKasirInduk', $data['id_kasir_induk']);
			$this->db->order_by("Nama", "asc");
			$result = $this->db->get();
			
			// Find sub agen dulu
			foreach($result->result() as $key=>&$item){
				$this->db->from('players');
				$this->db->where('IdKasir', $item->IdKasir);
				$player_results = $this->db->get(); 
				$item->AnakSub = $player_results->result();
			}

			return $result->result();
		}

		// Hanya bisa jalan untuk level 1, test function untuk cari kasir dengan data yang banyak
		public function get_agen_join(){
			$this->db->from('kasir as ka');
			$this->db->select('kb.IdKasir, kb.Nama, kb.level');

			$this->db->where('ka.IdKasirInduk', 230);
			$this->db->where('ka.IdKasir != kb.IdKasir');

			$this->db->join('kasir as kb', 'kb.IdKasirInduk = ka.IdKasir');

			$result = $this->db->get();
			return $result->result();
		}

		public function get_all_master_list(){
			$this->db->from('kasir');
			$this->db->where('level', '1');

			$this->db->select('kasir.IdKasir, kasir.Nama');

			$result = $this->db->get();
			return $result->result();
		}

		public function get_all_masterid_flatten(){
			$this->db->from('kasir');
			$this->db->where('level', '1');
			$this->db->select('kasir.IdKasir');

			$result = $this->flatten($this->db->get()->result_array());

			return $result;	
		}

		public function get_grup($data){
			$this->db->from('kasiraccounts');
			$this->db->where('IdKasir', $data);
			$this->db->order_by('NamaAccount', 'asc');

			$result = $this->db->get();
			return $result->result();
		}


		public function get_formatted_bank_date(){
			$this->db->from('bank');
			$this->db->distinct('Waktu');

			$this->db->select("DATE(Waktu) AS Waktu");
			$this->db->where('Jumlah != ', 0);
			$this->db->where('SaldoAwal != ', 0);
			$this->db->where('SaldoAkhir != ', 0);
			$result = $this->db->get();
			return $result->result();
		}

		// Rollingan harian tanpa range, mungkin boleh dikasih support range
		public function find_rollingan_harian($data, $subagen_list = ''){
			$subagen_list = $this->get_all_subagen($data);

			$this->db->from('memplayers AS mem');
			$this->db->group_by('mem.IdAccount');

			// tambahkan support cek pemain level 2, dan 1.
			// penanganan user level 2  dan 1 sama.
			
			// Untuk ambil subagen
			$this->db->where_in('mem.IdKasir', $subagen_list);

			$this->db->where('mem.IdOperation', $data['id_operation']);

			// Untuk ambil player yang langsung sambung ke subagen.
			// $this->db->where('mem.IdKasir', $data['id_kasir_induk']);
	
			// Harus di cek ke table accounts terlebih dahulu
			$this->db->join('accounts', 'accounts.IdAccount = mem.IdAccount');
			$this->db->join('players', 'players.IdPlayer = accounts.IdPlayer');
			$this->db->join('kasir', 'kasir.IdKasir = mem.IdKasir', 'left');

			$result = $this->db->get();

			return $result->result();
		}

		public function find_rollingan_range_sum_master($data){
			$this->db->from('kasir as ka');
			$this->db->select(
				'SUM(mem.CoinIn) as CoinInSum, SUM(mem.CoinOut) as CoinOutSum, 
				SUM(mem.DepositIn) as DepositInSum, SUM(mem.DepositOut) as DepositOutSum, 
				SUM(mem.TransferIn) as TransferInSum, 
				SUM(mem.TransferOut) as TransferOutSum, SUM(mem.Bet) as BetSum,
				SUM(mem.WinBet) as WinBetSum, SUM(mem.SaldoAwal) as SaldoAwalSum,
				SUM(mem.SaldoAkhir) as SaldoAkhirSum,
				mem.IdKasir, kb.Nama as NamaKasir, kb.IdKasirInduk, ka.Nama AS NamaKasirInduk,
				accounts.IdAccount, accounts.IdPlayer'
			);
			// Untuk local development pakai query ini
			// ANY_VALUE(accounts.IdAccount), ANY_VALUE(accounts.IdPlayer)'

			$this->db->group_by('mem.IdKasir');

			// $this->db->where_in('ka.IdKasirInduk', $data['master_id_list']);
			
			// Sudah ganti pakai teknik JOIN, sudah bisa handle data banyak
			$this->db->where('ka.IdKasirInduk', $data['id_kasir_induk']);
			$this->db->where('ka.IdKasir != kb.IdKasir');
			
			$this->db->where('IdOperation >=', $data['id_operation_from']);
			$this->db->where('IdOperation <=', $data['id_operation_to']);

			$this->db->order_by("NamaKasirInduk", "asc");

			$this->db->join('kasir as kb', 'kb.IdKasirInduk = ka.IdKasir');
			$this->db->join('memplayers as mem', 'mem.IdKasir = kb.IdKasir');
			$this->db->join('accounts', 'accounts.IdAccount = mem.IdAccount');
			
			$result = $this->db->get();

			if($result->num_rows() > 0){
				return $result->result();	
			}			
		}

		public function find_rollingan_range_sum($data){
			$this->db->from('kasir as ka');
			$this->db->select(
				'SUM(mem.CoinIn) as CoinInSum, SUM(mem.CoinOut) as CoinOutSum, 
				SUM(mem.DepositIn) as DepositInSum, SUM(mem.DepositOut) as DepositOutSum, 
				SUM(mem.TransferIn) as TransferInSum, 
				SUM(mem.TransferOut) as TransferOutSum, SUM(mem.Bet) as BetSum,
				SUM(mem.WinBet) as WinBetSum, SUM(mem.SaldoAwal) as SaldoAwalSum,
				SUM(mem.SaldoAkhir) as SaldoAkhirSum,
				mem.IdKasir, kb.Nama as NamaKasir, kb.IdKasirInduk, ka.Nama AS NamaKasirInduk,
				accounts.IdAccount, accounts.IdPlayer'
			);

			$this->db->group_by('mem.IdKasir');

			// Sudah ganti pakai teknik JOIN, sudah bisa handle data banyak
			$this->db->where('ka.IdKasirInduk', $data['id_kasir_induk']);
			$this->db->where('ka.IdKasir != kb.IdKasir');
			
			$this->db->where('IdOperation >=', $data['id_operation_from']);
			$this->db->where('IdOperation <=', $data['id_operation_to']);

			$this->db->order_by("NamaKasirInduk", "asc");

			$this->db->join('kasir as kb', 'kb.IdKasirInduk = ka.IdKasir');
			$this->db->join('memplayers as mem', 'mem.IdKasir = kb.IdKasir');
			$this->db->join('accounts', 'accounts.IdAccount = mem.IdAccount');
			
			$result = $this->db->get();
			
			return $result->result();
		}

		public function find_rollingan_range_sum_lv2($data, $subagen_list){
			$this->db->from('memplayers AS mem');

			$this->db->select('kasir.Nama as NamaKasir, kasir.IdKasirInduk as IdKasirInduk,
				SUM(mem.CoinIn) as CoinInSum, SUM(mem.CoinOut) as CoinOutSum,
				SUM(mem.DepositIn) as DepositInSum, SUM(mem.DepositOut) as DepositOutSum,
				SUM(mem.TransferIn) as TransferInSum, mem.IdKasir as IdKasir, 
				SUM(mem.TransferOut) as TransferOutSum, SUM(mem.Bet) as BetSum,
				SUM(mem.WinBet) as WinBetSum, SUM(mem.SaldoAwal) as SaldoAwalSum,
				SUM(mem.SaldoAkhir) as SaldoAkhirSum, players.NickName as NickName,
				machines.CreditAkhir, machines.TipeBet, global.SaldoAkhir as SaldoAkhirPlayer'
			);

			$this->db->where_in('mem.IdKasir', $subagen_list);
			$this->db->where('IdOperation >=', $data['id_operation_from']);
			$this->db->where('IdOperation <=', $data['id_operation_to']);
			
			$this->db->group_by('mem.IdAccount');

			$this->db->join('accounts', 'accounts.IdAccount = mem.IdAccount');
			$this->db->join('kasir', 'kasir.IdKasir = mem.IdKasir');
			$this->db->join('players', 'players.IdPlayer = accounts.IdPlayer');
			$this->db->join('machines_players as machines', 'machines.IdPlayer = accounts.IdPlayer', 'left');
			$this->db->join('globalmemplayers as global', 'global.IdAccount = accounts.IdAccount', 'left');

			$result = $this->db->get();

			return $result->result();
		}

		public function find_rollingan_range_sum_lv3($data){
			$this->db->from('memplayers AS mem');
			$this->db->select(
				'SUM(mem.CoinIn) as CoinInSum, SUM(mem.CoinOut) as CoinOutSum, 
				SUM(mem.DepositIn) as DepositInSum, SUM(mem.DepositOut) as DepositOutSum, 
				SUM(mem.TransferIn) as TransferInSum, 
				SUM(mem.TransferOut) as TransferOutSum, SUM(mem.Bet) as BetSum,
				SUM(mem.WinBet) as WinBetSum, SUM(mem.SaldoAwal) as SaldoAwalSum,
				SUM(mem.SaldoAkhir) as SaldoAkhirSum,
				mem.IdKasir, user.Nama as NamaKasir, user.IdKasirInduk, players.NickName,
				machines.CreditAkhir, machines.TipeBet, global.SaldoAkhir as SaldoAkhirPlayer'
			);

			$this->db->group_by('mem.IdAccount');
			$this->db->where('mem.IdKasir', $data['id_kasir_induk']);
			$this->db->where('IdOperation >=', $data['id_operation_from']);
			$this->db->where('IdOperation <=', $data['id_operation_to']);

			$this->db->join('accounts', 'accounts.IdAccount = mem.IdAccount');
			$this->db->join('kasir AS user', 'user.IdKasir = mem.IdKasir');
			$this->db->join('players', 'players.IdPlayer = accounts.IdPlayer');
			$this->db->join('machines_players as machines', 'machines.IdPlayer = accounts.IdPlayer', 'left');
			$this->db->join('globalmemplayers as global', 'global.IdAccount = accounts.IdAccount', 'left');

			$result = $this->db->get();

			return $result->result();
		}

		public function test_q($data){
			$this->db->from('memplayers AS mem');
			$this->db->select(
				'SUM(mem.CoinIn) as CoinInSum, SUM(mem.CoinOut) as CoinOutSum, 
				SUM(mem.DepositIn) as DepositInSum, SUM(mem.DepositOut) as DepositOutSum, 
				SUM(mem.TransferIn) as TransferInSum, 
				SUM(mem.TransferOut) as TransferOutSum, SUM(mem.Bet) as BetSum,
				SUM(mem.WinBet) as WinBetSum, SUM(mem.SaldoAwal) as SaldoAwalSum,
				SUM(mem.SaldoAkhir) as SaldoAkhirSum,
				mem.IdKasir, user.Nama as NamaKasir, user.IdKasirInduk, players.NickName,
				accounts.IdAccount, accounts.IdPlayer'
			);

			$this->db->group_by('mem.IdAccount');
			$this->db->where('mem.IdKasir', "1");
			$this->db->where('IdOperation >=', "2");
			$this->db->where('IdOperation <=', "3");

			$this->db->join('accounts', 'accounts.IdAccount = mem.IdAccount');
			$this->db->join('kasir AS user', 'user.IdKasir = mem.IdKasir', 'left');
			$this->db->join('players', 'players.IdPlayer = accounts.IdAccount', 'left');
			
			$result = $this->db->get();

			return $result->result();
		}

		// TODO: (1) perlu diganti dengan query yang lebih cepat
		public function find_rollingan_global($data = ''){
			$this->db->from('globalmemplayers as gm');

			$this->db->select("Bet, ContactName, CreditAkhir, DepositIn, 
				DepositOut, FreeCoin, Handphone, accounts.IdAccount, IdKasir, 
				IdKasirAccount, IdMachine, players.IdPlayer, NamaBank, NickName, 
				NomorAccount, Password, SaldoAkhir, TipeBet, TransferIn, 
				TransferOut, WaktuCipta, WinBet");

			if ($data !== '') {
				$this->db->where_in('gm.IdAccount', $data['id_account']);
			}
			$this->db->order_by('players.NickName', 'asc'); 
			
			$this->db->join('accounts', 'accounts.IdAccount = gm.IdAccount');
			$this->db->join('players', 'players.IdPlayer = accounts.IdPlayer');
			$this->db->join('machines_players as mp', 'mp.IdPlayer = players.IdPlayer', 'left');

			$result = $this->db->get();
			return $result->result();
		}

		public function all_player(){
			$this->db->from('players');
			$result = $this->db->get();
			return $result->result();
		}

		public function get_operation_list(){
			$this->db->from('operations');
			$this->db->select("IdOperation, WaktuTutup, WaktuBuka");
			// Comment out sementara untuk show jam
			// $this->db->select("IdOperation, WaktuTutup, DATE_FORMAT(WaktuBuka, ' %d %M %Y') AS WaktuBuka");
			$this->db->order_by("IdOperation", "desc");
			
			$result = $this->db->get();
			return $result->result();
		}

		public function find_transfer_list($data){
			$this->db->from('transferpemain AS transfer');
			$this->db->where('IdOperation', $data['id_operation']);

			// kalau master lihat semua transferan berdasarkan periode, tidak lewat IF

			if ($data['id_level'] == 2) { // if level 2, cari berdasarkan ID subagen
				// print_r($data['subagen_lst']);
				$this->db->where_in('transfer.IdKasir', $data['subagen_lst']);
			}else if($data['id_level'] == 3){ // if level 3, langsung cari berdasarkan ID sekarang
				$this->db->where('transfer.IdKasir', $data['id_kasir_induk']);
			}
			$this->db->join('kasir', 'kasir.IdKasir = transfer.IdKasir', 'left');

			$result = $this->db->get();
			return $result->result();
		}

		public function find_player_machine($data){
			$this->db->from('history_games');
			$this->db->select('history_games.IdMachine, history_games.IdPlayer, NickName');
			$this->db->distinct();
			$this->db->where('IdOperation', $data['id_operation']);

			// IF untuk handle agen level 2
			if ($data['id_level'] != 1) {
				$this->db->where_in('players.IdPlayer', $data['player_lst']);	
			}

			$this->db->join('players', 'players.IdPlayer = history_games.IdPlayer', 'left');
			$result = $this->db->get();
			return $result->result();
		}

		public function find_player_machine_saldo($data){
			$this->db->from('history_games');
			$this->db->where('IdOperation', $data['id_operation']);
			$this->db->where('IdPlayer', $data['player_id']);
			$result = $this->db->get();
			return $result->result();
		}

		public function find_machine($data){
			$this->db->from('history_games');
			$this->db->select('IdMachine');
			$this->db->where('IdPlayer', $data['player_id']);
			$this->db->group_by('IdMachine'); 
			$this->db->order_by('IdMachine', 'asc'); 

			$result = $this->db->get();
			
			return $result->result_array();	
		}

		public function find_games($data){
			$this->db->from('history_games as g');
			$this->db->where('g.IdPlayer', $data['player_id']);
			$this->db->where('g.IdMachine', $data['machine_id']);

			$this->db->join('tipe_jadi as tipe', 'tipe.kode_jadi = g.IdJadi', 'left');
			$this->db->join('tipe_jadi_extra as tipe_extra', 'tipe_extra.kode_jadi_extra = g.IdJadiExtra', 'left');
		
			$this->db->order_by('Waktu', 'asc'); 

			$result = $this->db->get();
			return $result->result_array();	
		}

		public function find_kasir_account($data){
			$this->db->from('kasiraccounts as k');

			$this->db->where('IdKasir', $data['id_kasir_induk']);
			$this->db->order_by("NamaAccount", "asc");

			$result = $this->db->get();
			return $result->result();
		}

		public function find_kasiraccount_player($data){
			$this->db->from('players as p');

			$this->db->where_in('IdKasir', $data);
			$this->db->order_by("NickName", "asc");
			$this->db->join('accounts', 'accounts.IdPlayer = p.IdPlayer');

			$result = $this->db->get();
			return $result->result();
		}

		public function find_deposit_in($data){
			$this->db->from('bank as b');

			// Tambahkan select, karena banyak field yang namanya sama
			$this->db->select(
				"b.IdTransaksi, kasiraccounts.NamaAccount as NamaAccountKasir, players.IdPlayer as IdPlayer,
				players.NickName as NickName, players.ContactName as ContactName, 
				players.Handphone as Handphone, b.SaldoAwal, b.Jumlah, b.SaldoAkhir, DATE_FORMAT(b.Waktu, ' %d-%m-%Y %H:%i:%s') AS Waktu,
				accounts.NomorAccount as NomorAccountPlayer, accounts.NamaBank as NamaBankPlayer,
			");

			$this->db->order_by("b.IdTransaksi", "desc");

			// Where from date
			$this->db->where('DATE_FORMAT(b.Waktu, "%Y-%m-%d") >=', $data['deposit_date_from']);

			// To date
			$this->db->where('DATE_FORMAT(b.Waktu, "%Y-%m-%d") <=', $data['deposit_date_to']);

			$this->db->where('players.IdKasir', $data['subagen_id']);

			// Cek data yang 0, untuk mempercepat query
			$this->db->where('SaldoAwal !=',0);
			$this->db->where('Jumlah !=', 0);
			$this->db->where('SaldoAkhir !=', 0);

			// Cari kategori deposit 
			$this->db->where('IdCategory', $data['tipe_deposit']);

			// Join dengan Table account untuk ambil no rek
			$this->db->join('accounts', 'accounts.IdAccount = b.IdAccount', 'left');

			// Join dengan table player untuk ambil nickname
			$this->db->join('players', 'players.IdPlayer = accounts.IdPlayer', 'left');

			// Join dengan kasir account untuk tau nama account kasir
			$this->db->join('kasiraccounts', 'kasiraccounts.IdAccount = players.IdKasirAccount');

			$result = $this->db->get();
			return $result->result();

		}

		public function find_isi_coin($data){
			$this->db->from('bank_kasir as bk');

			$this->db->select(
				"bk.IdTransaksi as IdTransaksiFrom, bk.IdKasir as IdKasirFrom, bk.SaldoAwal as SaldoAwalFrom, 
				bk.Jumlah as JumlahFrom, bk.SaldoAkhir as SaldoAkhirFrom, bk.IdCategory as IdCategoryFrom,
				bk.IdTransaksiKasir as IdTransaksiKasirFrom, DATE_FORMAT(bk.Waktu, ' %d-%m-%Y %H:%i:%s') AS WaktuFrom, 
				ks.Nama as NamaFrom, bs.IdTransaksi as IdTransaksiTo, bs.IdKasir as IdKasirTo, ka.Nama as NamaTo, bs.SaldoAwal as SaldoAwalTo,
				bs.SaldoAkhir as SaldoAkhirTo, bs.IdCategory as IdCategoryTo
			");

			$this->db->order_by("bk.IdTransaksiKasir", "desc");

			// Where from date
			$this->db->where('DATE_FORMAT(bk.Waktu, "%Y-%m-%d") >=', $data['deposit_date_from']);

			// To date
			$this->db->where('DATE_FORMAT(bk.Waktu, "%Y-%m-%d") <=', $data['deposit_date_to']);

			// Cari berdasarkan pengirim
			$this->db->where('bk.IdKasir', $data['agen_id']);
			$this->db->where('bk.IdKasir != bs.IdKasir');

			$this->db->where_in('bk.IdCategory', array(58));

			$this->db->join('bank_kasir as bs', 'bs.IdTransaksiKasir = bk.IdTransaksiKasir');
			$this->db->join('kasir as ks', 'ks.IdKasir = bk.IdKasir');
			$this->db->join('kasir as ka', 'ka.IdKasir = bs.IdKasir');

			$result = $this->db->get();
			return $result->result();
		}

		public function find_cancel_deposit($data){
			$this->db->from('bank_kasir as bk');

			$this->db->select(
				"bk.IdTransaksi as IdTransaksiFrom, bk.IdKasir as IdKasirFrom, bk.SaldoAwal as SaldoAwalFrom, 
				bk.Jumlah as JumlahFrom, bk.SaldoAkhir as SaldoAkhirFrom, bk.IdCategory as IdCategoryFrom,
				bk.IdTransaksiKasir as IdTransaksiKasirFrom, DATE_FORMAT(bk.Waktu, ' %d-%m-%Y %H:%i:%s') AS WaktuFrom, 
				ks.Nama as NamaFrom, bs.IdTransaksi as IdTransaksiTo, bs.IdKasir as IdKasirTo, ka.Nama as NamaTo, bs.SaldoAwal as SaldoAwalTo,
				bs.SaldoAkhir as SaldoAkhirTo, bs.IdCategory as IdCategoryTo
			");

			$this->db->order_by("bk.IdTransaksiKasir", "desc");

			// Where from date
			$this->db->where('DATE_FORMAT(bk.Waktu, "%Y-%m-%d") >=', $data['deposit_date_from']);

			// To date
			$this->db->where('DATE_FORMAT(bk.Waktu, "%Y-%m-%d") <=', $data['deposit_date_to']);

			// Cari berdasarkan pengirim
			$this->db->where('bk.IdKasir', $data['agen_id']);
			$this->db->where('bk.IdKasir != bs.IdKasir');

			$this->db->where_in('bk.IdCategory', array(59));

			$this->db->join('bank_kasir as bs', 'bs.IdTransaksiKasir = bk.IdTransaksiKasir');
			$this->db->join('kasir as ks', 'ks.IdKasir = bk.IdKasir');
			$this->db->join('kasir as ka', 'ka.IdKasir = bs.IdKasir');

			$result = $this->db->get();
			return $result->result();
		}

		public function flatten(array $array) {
		    $return = array();
		    array_walk_recursive($array, function($a) use (&$return) { $return[] = $a; });
		    return $return;
		}

		public function login_history($data){
			$this->db->from('log_connection_players as history');

			if ($data['player_id'] != FALSE) {
				$this->db->where('history.IdPlayer', $data['player_id']);	
			}else{
				$this->db->where_in('history.IdPlayer', $data['player_lst']);
			}

			$this->db->where('IdOperation', $data['id_operation']);
			$this->db->join('players', 'players.IdPlayer = history.IdPlayer', 'left');
			$result = $this->db->get();
			
			return $result->result_array();	
		}
	}

?>